<?php

namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DataSource\Manager;
use Teknisa\MAN\Services;
use Teknisa\MAN\Utilities;

use \Exception as Exception;


class Evento extends CrudWithParameterBag {
    const dataSourceEventos = 'select_eventos';
    const dataSourceUnidades = 'unidades_participantes_eventos';
    const dataSourceCaixas = 'caixas_participantes_eventos';
    const dataSourceSelectCaixasEventos = 'select_caixas_eventos';
    
      
    protected $parameterBagColumns = ['NRORG'];
    protected $dataSourceManager;
    protected $eventoService; 
    protected $environment;
    protected $util;
 
    public function __construct (ParameterBag $parameterBag, Manager $dataSourceManager, Services\EventoService $EventoService, Utilities\Util $util, $environment) {
        $this->util = $util;
        $this->parameterBag = $parameterBag;
        $this->dataSourceManager = $dataSourceManager;
        $this->EventoService = $EventoService;
        $this->environment = $environment;
    }
        
    public function findEvento(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceEventos;
        parent::find($request, $response);
    }
        
    public function saveEvento(DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = self::dataSourceEventos;
        $dataSet = $request->getDataSet();
        $rows = $dataSet->getRows();
        
        try {
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceName);
            $persistedRows = $this->dataSourceManager->persist($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $persistedRows));
            $nrPersistedRows = count($persistedRows);
            
        } catch (Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
        
    public function deleteEvento (DTO\Request\DataSet $request,DTO\Response $response) {
    	try {
    	    $this->dataSourceName = 'select_eventos';
    	    $params = $this->util->getParamsAsArray($request);
    	    $params['NRORG'] = $this->environment->getNrOrgTrab();
            $this->EventoService->deleteEventoCascade($params[0]['CDEVENTO'], $params['NRORG']);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
          
    public function findUnidadeEvento(DTO\Request\Filter $request, DTO\Response $response){
               
        $this->dataSourceName = self::dataSourceUnidades;
        $this->parameterBagColumns = array('NRORG', 'CDEVENTO');
        parent::find($request, $response);
    }
        
    public function saveUnidadeEvento(DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceUnidades;
        $dataSet = $request->getDataSet();
        $rows = $dataSet->getRows();
        
        try {
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceName);
            $persistedRows = $this->dataSourceManager->persist($dataSet);
            
            $response->addDataSet(new DataSet($this->dataSourceName, $persistedRows));
            $nrPersistedRows = count($persistedRows);
            
        } catch (Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
        
    public function deleteUnidadeEvento (DTO\Request\DataSet $request,DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as $row) {
                $params['NRORG'] = $this->environment->getNrOrgTrab();
                $this->EventoService->deleteCascade($row['CDFILIAL'], $row['CDEVENTO'], $params['NRORG']);
            }
            
    	    $this->dataSourceName = 'unidades_participantes_eventos';
            parent::delete($request, $response);
           
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
        
    public function findCaixaEvento (DTO\Request\Filter $request, DTO\Response $response){
           
        $this->dataSourceName = self::dataSourceCaixas;
        $this->parameterBagColumns = array('NRORG', 'CDFILIAL','CDEVENTO');
        parent::find($request, $response);
    }
        
    public function saveCaixaEvento (DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceCaixas;
        
        try {
            foreach ($request->getDataSet()->getRows() as $row) {
                $arrayCodigoCaixa = $row['CDCAIXA'];
                $row['NMCAIXA'] = $row['NMCAIXA'];
                
                foreach ($arrayCodigoCaixa as $caixa) {
                    $row['CDCAIXA'] = $caixa;
                    $row['__is_new'] = true;
                    $row['changed'] = false;
                    $this->dataSourceManager->persist(new DataSet($this->dataSourceName, array($row)));
                }
            }
            
            $response->addNotification(new Notification("Dados inseridos com sucesso!", Notification::TYPE_SUCCESS));
        } catch (Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
        
    public function deleteCaixaEvento (DTO\Request\DataSet $request,DTO\Response $response) {
    	try {
    	    $this->dataSourceName = 'caixas_participantes_eventos';
            parent::delete($request,$response);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
}